import static gdbutil.ConnectionParameters.*;

import java.awt.geom.Point2D;
import java.sql.*;

import oracle.spatial.geometry.JGeometry;  // sdoapi.jar
import oracle.spatial.network.*;           // sdonm.jar

/**
 * Klasse, um eine Netz-Topologie zu ergnzen und auszuwerten.
 * Fr Oracle Spatial 10.
 * 
 * @author: Thomas Brinkhoff, http://www.geodbs.de/
 */
public class NetTopology {

	/**
	 * Konstruktor.
	 */
	public NetTopology() {
		super();
	}

	/**
	 * Berechnung und Speicherung von Wegen. 
	 */
	public void computePaths () {
		try {
			// Verbindung zur Datenbank aufbauen
			Class.forName("oracle.jdbc.driver.OracleDriver");
			String url = "jdbc:oracle:thin:@" + DBHOST + ":" + DBPORT + ":" + DBNAME;  // JDBC Thin Driver
				// bei JDBC OCI Driver:  "jdbc:oracle:oci:@" + DBSERVICENAME;
			Connection db = DriverManager.getConnection(url, DBUSER, DBPASSWORD);
			PreparedStatement anw = db.prepareStatement(
			"INSERT INTO OldenburgNetz_PATH$ (path_id, start_node_id, end_node_id, cost, geometry) VALUES (?,?,?,?,?)");
			// Wege berechnen
			Network net = NetworkManager.readNetwork(db,"OldenburgNetz");
			Path paths[] = NetworkManager.withinCost(net,6436,10000.0);
			//Path paths[] = NetworkManager.nearestNeighbors(net,6436,40);
			// Fr alle Wege:
			for (Path path : paths) {
				// Geometrie berechnen und Pfad speichern
				path.computeGeometry(0.5);
				net.addPath(path);
			}
			// Netzwerk speichern
			NetworkManager.writeNetwork(db,net);
			// Verbindung schlieen
			anw.close();
			db.close();
			System.out.println("fertig!");
		}
		// Falls Fehler auftritt:
		catch (Exception ex) {
			System.err.println("NetTopology.computePaths: "+ex);
			ex.printStackTrace();
		}
	}
	
	/**
	 * Berechnung und Speicherung des krzesten Wegs. 
	 */
	public void computeShortestPath () {
		try {
			// Verbindung zur Datenbank aufbauen
			Class.forName("oracle.jdbc.driver.OracleDriver");
			String url = "jdbc:oracle:thin:@" + DBHOST + ":" + DBPORT + ":" + DBNAME;  // JDBC Thin Driver
				// bei JDBC OCI Driver:  "jdbc:oracle:oci:@" + DBSERVICENAME;
			Connection db = DriverManager.getConnection(url, DBUSER, DBPASSWORD);
			// Krzesten Weg berechnen
			Network net = NetworkManager.readNetwork(db,"OldenburgNetz",true);
			Path path = NetworkManager.shortestPathDijkstra(net,1345,6436);
			//Path path = NetworkManager.shortestPathAStar(net,1345,6436);
			//Path path = NetworkManager.tspPath(net,new int[]{1345,2222,3234,4999,5345,6436},true,true,null);
			// Geometrie berechnen
			path.computeGeometry(0.5);
			// Pfad und Netzwerk speichern
			net.addPath(path);
			NetworkManager.writeNetwork(db,net);
			// Verbindung schliessen
			db.close();
			System.out.println("fertig!");
		}
		// Falls Fehler auftritt:
		catch (Exception ex) {
			System.err.println("NetTopology.computeShortestPath: "+ex);
			ex.printStackTrace();
		}
	}
	
	/**
	 * Ergnze die Kanten um die Geometrie und Kosten. 
	 */
	public void completeEdges () {
		try {
			// Verbindung zur Datenbank aufbauen
			//Class.forName("oracle.jdbc.driver.OracleDriver");
			String url = "jdbc:oracle:thin:@" + DBHOST + ":" + DBPORT + ":" + DBNAME;  // JDBC Thin Driver
				// bei JDBC OCI Driver:  "jdbc:oracle:oci:@" + DBSERVICENAME;
			Connection db = DriverManager.getConnection(url, DBUSER, DBPASSWORD);
			// Anfragen vorbereiten
			Statement linkQuery = db.createStatement();
			Statement nodeQuery = db.createStatement();
			PreparedStatement update = db.prepareStatement("UPDATE OldenburgNetz_LINK$ SET geometry = ?, cost = ? WHERE link_id = ?");
			ResultSet rs = linkQuery.executeQuery("SELECT link_id,start_node_id,end_node_id,link_type FROM OldenburgNetz_LINK$");
			// Alle Kanten durchlaufen
			while (rs.next()) {
				int id = rs.getInt("LINK_ID");
				System.out.println(id);
				// Anfangsknoten abfragen
				ResultSet rs1 = nodeQuery.executeQuery("SELECT geometry FROM OldenburgNetz_NODE$ WHERE node_id = "+rs.getInt("start_node_id"));
				rs1.next();
				Struct struct = (Struct) rs1.getObject(1);
				JGeometry jGeom = JGeometry.loadJS(struct);
				Point2D p1 = jGeom.getJavaPoint();
				rs1.close();
				// Endknoten abfragen
				ResultSet rs2 = nodeQuery.executeQuery("SELECT geometry FROM OldenburgNetz_NODE$ WHERE node_id = "+rs.getInt("end_node_id"));
				rs2.next();
				struct = (Struct) rs2.getObject(1);
				jGeom = JGeometry.loadJS(struct);
				Point2D p2 = jGeom.getJavaPoint();
				rs2.close();
				// Kantengeometrie erzeugen, Kosten berechnen und speichern
				double[] pkt = new double[] {p1.getX(),p1.getY(),p2.getX(),p2.getY()};
				JGeometry kante = JGeometry.createLinearLineString(pkt,2,0);
				update.setObject(1,JGeometry.storeJS(kante,db));
				update.setDouble(2,p1.distance(p2)*rs.getInt("link_type"));
				update.setInt(3,id);
				update.execute();
			}
			// ResultSet und Statements schlieen
			rs.close();
			nodeQuery.close();
			linkQuery.close();
			update.close();
			// Verbindung schliessen
			db.close();
			System.out.println("fertig!");
		}
		// Falls Fehler auftritt:
		catch (Exception ex) {
			System.err.println("NetTopology.completeEdges: "+ex);
			ex.printStackTrace();
		}

	}
	
	/**
	 * Main method: ruft Methoden auf.
	 */
	public static void main (String[] args) {
			//new NetTopology().completeEdges();
			//new NetTopology().computePaths();
			new NetTopology().computeShortestPath();
	}
	
}
